/*

					Oscilloscope AVR

			Copyrights 2008,  Yoshi Hiroshi

		Control Block - ATmega644P/ATmega324P

*/

#include <avr/io.h>

#include "OscilloCtrl.h"
#include "Sampling.h"
#include "AdcManager.h"
#include "Spi.h"
#include "Buffer.h"
#include "Uart.h"

//**********************************************************
//		Reset All
//**********************************************************
//
//----------------------------------------------------------
void ResetAll(rxBuffType* ptr)
{
	PutFreeRxBuff(ptr);
	TIMING_GEN_CMND_PORT = 0;	
	SamplingStop(STOP_REQUEST_ON);
	ADC_CLEAR_ALL;
	txBuffType* txPtr;
	while((txPtr = GetUsedTxBuff(GET_BUFFER_NOWAIT)) != 0)
	{
		PutFreeTxBuff(txPtr);
	}
}

//**********************************************************
//		GetConfiguration
//**********************************************************
//
//----------------------------------------------------------
void GetConfiguration(rxBuffType* ptr)
{
	PutFreeRxBuff(ptr);
	CheckConfiguration();
	txBuffType* sPtr = GetFreeTxBuff(GET_BUFFER_WAIT);
	MakeReplyConfiguration(sPtr->buff, GetNumChannels(), GetNumADCs(), GetVref(), GetSlowestPeriodCode(), GetFastestPeriodCode(), GetBuffSize() * GetNumADCs());
	SendTxBuff(sPtr);
}

//**********************************************************
//		StartSampling
//**********************************************************
//
//----------------------------------------------------------
void StartSampling(rxBuffType* ptr)
{
	samplePeriodCode =  ptr->buff[2];
	triggerMode = ptr->buff[3];
	triggerLevel = ptr->buff[4];
	delayUnit = ptr->buff[5];
	delayTime = ((ptr->buff[6] & 0xFF) << 8) | (ptr->buff[7] & 0xFF);
	numSampleData = (ptr->buff[8] & 0xFF);
	numSampleData = ((numSampleData << 8) & 0xFF00) | (ptr->buff[9] & 0xFF);
	numSampleData = ((numSampleData << 8) & 0xFFFF00) | (ptr->buff[10] & 0xFF);
	PutFreeRxBuff(ptr);
	triggerEdge = 0;
	triggerChannel = 0;
	if((triggerMode & TRIGGER_MODE_CHANNEL_1) != 0) triggerChannel = 1;
	if((triggerMode & TRIGGER_MODE_CHANNEL_2) != 0) triggerChannel = 2;
	if((triggerMode & TRIGGER_MODE_PLUS) != 0) triggerEdge = TRIGGER_MODE_PLUS;
	if((triggerMode & TRIGGER_MODE_MINUS) != 0) triggerEdge = TRIGGER_MODE_MINUS;
	if((triggerMode & TRIGGER_MODE_DELAYED) == 0) delayTime = 0;
	if((triggerMode & TRIGGER_MODE_AUTO) != 0) 
	{
		triggerMode = TRIGGER_MODE_AUTO;
	} else {
		triggerMode = 0;
	}
	SamplingStart();
}

//**********************************************************
//		StopSampling
//**********************************************************
//
//----------------------------------------------------------
void StopSampling(rxBuffType* ptr)
{
	PutFreeRxBuff(ptr);
	SamplingStop(STOP_REQUEST_ON);
	txBuffType* sPtr;
	while((sPtr = GetUsedTxBuff(GET_BUFFER_NOWAIT)) != 0)
	{
		PutFreeTxBuff(sPtr);
	}
	sPtr = GetFreeTxBuff(GET_BUFFER_WAIT);
	MakeReplyStopSampling(sPtr->buff, numSampleData);
	SendTxBuff(sPtr);
}

//**********************************************************
//		SetSettings
//**********************************************************
//
//----------------------------------------------------------
void SetSettings(rxBuffType* ptr)
{
	unsigned char range1 = ptr->buff[2];
	unsigned char pos1 = ptr->buff[3];
	unsigned char sw1 = ptr->buff[4];
	unsigned char range2 = ptr->buff[5];
	unsigned char pos2 = ptr->buff[6];
	unsigned char sw2 = ptr->buff[7];
	PutFreeRxBuff(ptr);
	SetAdcRange(1, range1);
	SetAdcPosition(1, pos1);
	SetAdcSwitch(1, sw1);
	SetAdcRange(2, range2);
	SetAdcPosition(2, pos2);
	SetAdcSwitch(2, sw2);
	txBuffType* sPtr = GetFreeTxBuff(GET_BUFFER_WAIT);
	MakeReplySetSettings(sPtr->buff, 0x00);
	SendTxBuff(sPtr);
}
//**********************************************************
//		SetRange
//**********************************************************
//
//----------------------------------------------------------
void SetRange(rxBuffType* ptr)
{
	unsigned char ch = ptr->buff[2];
	unsigned char val = ptr->buff[3];
	PutFreeRxBuff(ptr);
	SetAdcRange(ch, val);
	txBuffType* sPtr = GetFreeTxBuff(GET_BUFFER_WAIT);
	MakeReplySetRange(sPtr->buff, ch, val);
	SendTxBuff(sPtr);
}

//**********************************************************
//		SetPosition
//**********************************************************
//
//----------------------------------------------------------
void SetPosition(rxBuffType* ptr)
{
	unsigned char ch = ptr->buff[2];
	unsigned char val = ptr->buff[3];
	PutFreeRxBuff(ptr);
	SetAdcPosition(ch, val);
	txBuffType* sPtr = GetFreeTxBuff(GET_BUFFER_WAIT);
	MakeReplySetPosition(sPtr->buff, ch, val);
	SendTxBuff(sPtr);
}

//**********************************************************
//		SetTriggerMode
//**********************************************************
//
//----------------------------------------------------------
void SetTriggerMode(rxBuffType* ptr)
{
	unsigned char ch = ptr->buff[2];
	unsigned char val = ptr->buff[3];
	PutFreeRxBuff(ptr);
	SetAdcTriggerMode(ch, val);
	txBuffType* sPtr = GetFreeTxBuff(GET_BUFFER_WAIT);
	MakeReplySetTriggerMode(sPtr->buff, ch, val);
	SendTxBuff(sPtr);
}
//**********************************************************
//		SetTriggerLevel
//**********************************************************
//
//----------------------------------------------------------
void SetTriggerLevel(rxBuffType* ptr)
{
	unsigned char ch = ptr->buff[2];
	unsigned char val = ptr->buff[3];
	PutFreeRxBuff(ptr);
	SetAdcTriggerLevel(ch, val);
	txBuffType* sPtr = GetFreeTxBuff(GET_BUFFER_WAIT);
	MakeReplySetTriggerLevel(sPtr->buff, ch, val);
	SendTxBuff(sPtr);
}

//**********************************************************
//		SetSwitch
//**********************************************************
//
//----------------------------------------------------------
void SetSwitch(rxBuffType* ptr)
{
	unsigned char ch = ptr->buff[2];
	unsigned char val = ptr->buff[3];
	PutFreeRxBuff(ptr);
	SetAdcSwitch(ch, val);
	txBuffType* sPtr = GetFreeTxBuff(GET_BUFFER_WAIT);
	MakeReplySetSwitch(sPtr->buff, ch, val);
	SendTxBuff(sPtr);
}

//**********************************************************
//		ClearAll
//**********************************************************
//
//----------------------------------------------------------
void ClearAll(rxBuffType* ptr)
{
	unsigned char ch = ptr->buff[2];
	unsigned char un = ptr->buff[3];
	PutFreeRxBuff(ptr);
	ClearAdcAll(ch, un);
	txBuffType* sPtr = GetFreeTxBuff(GET_BUFFER_WAIT);
	MakeReplyClearAll(sPtr->buff, ch, un);
	SendTxBuff(sPtr);
}


//**********************************************************
//		GetNumData
//**********************************************************
//
//----------------------------------------------------------
void GetNumData(rxBuffType* ptr)
{
	unsigned char ch = ptr->buff[2];
	unsigned char un = ptr->buff[3];
	PutFreeRxBuff(ptr);
	unsigned int val = GetAdcNumData(ch, un);
	txBuffType* sPtr = GetFreeTxBuff(GET_BUFFER_WAIT);
	MakeReplyNumData(sPtr->buff, ch, un, val);
	SendTxBuff(sPtr);
}

//**********************************************************
//		GetFirstData
//**********************************************************
//
//----------------------------------------------------------
void GetFirstData(rxBuffType* ptr)
{
	unsigned char ch = ptr->buff[2];
	unsigned char un = ptr->buff[3];
	PutFreeRxBuff(ptr);
	unsigned char val = GetAdcFirstData(ch, un);
	txBuffType* sPtr = GetFreeTxBuff(GET_BUFFER_WAIT);
	MakeReplyFirstData(sPtr->buff, ch, un, val);
	SendTxBuff(sPtr);
}

//**********************************************************
//		GetNextData
//**********************************************************
//
//----------------------------------------------------------
void GetNextData(rxBuffType* ptr)
{
	unsigned char ch = ptr->buff[2];
	unsigned char un = ptr->buff[3];
	PutFreeRxBuff(ptr);
	unsigned char val = GetAdcNextData(ch, un);
	txBuffType* sPtr = GetFreeTxBuff(GET_BUFFER_WAIT);
	MakeReplyFirstData(sPtr->buff, ch, un, val);
	SendTxBuff(sPtr);
}

//**********************************************************
//		DoCommand
//**********************************************************
//
//----------------------------------------------------------
void DoCommand()
{
	rxBuffType* ptr = GetUsedRxBuff(GET_BUFFER_WAIT);
	unsigned char cmnd = ptr->buff[1];
	switch(cmnd)
	{
		case COMMAND_CODE_RESET_ALL:				// ResetAll
			ResetAll(ptr);
			break;
		case COMMAND_CODE_GET_CONFIGURATION:		// GetConfiguration
			GetConfiguration(ptr);
			break;
		case COMMAND_CODE_SET_SETTINGS:		// GetConfiguration
			SetSettings(ptr);
			break;
		case COMMAND_CODE_START_SAMPLING:			// StartSampling
			StartSampling(ptr);
			break;
		case COMMAND_CODE_STOP_SAMPLING:			// StopSampling
			StopSampling(ptr);
			break;

		case COMMAND_CODE_GET_DATA:					// GetData
			PutFreeRxBuff(ptr);
			break;

		case COMMAND_CODE_SET_RANGE:				// SetRange
			SetRange(ptr);
			break;
		case COMMAND_CODE_SET_POSITION:				// SetPosition
			SetPosition(ptr);
			break;
		case COMMAND_CODE_SET_TRIGGER_MODE:			// SetTriggerMode
			SetTriggerMode(ptr);
			break;
		case COMMAND_CODE_SET_TRIGGER_LEVEL:		// SetTriggerLevel
			SetTriggerLevel(ptr);
			break;
		case COMMAND_CODE_SET_SWITCH:				// SetSwitch
			SetSwitch(ptr);
			break;
		case COMMAND_CODE_CLEAR_ALL:				// ClearAll
			ClearAll(ptr);
			break;
		case COMMAND_CODE_GET_NUM_DATA:				// GetNumData
			GetNumData(ptr);
			break;
		case COMMAND_CODE_GET_FIRST_DATA:			// GetFirstData
			GetFirstData(ptr);
			break;
		case COMMAND_CODE_GET_NEXT_DATA:			// GetNextData
			GetNextData(ptr);
			break;
		default:
			PutFreeRxBuff(ptr);
			break;
	}
}
