/*


					Oscilloscope AVR

			Copyrights 2008,  Yoshi Hiroshi

		Control Block - ATmega644P/ATmega324P

*/

#include <avr/io.h>
#include <avr/interrupt.h>

#include "OscilloCtrl.h"
#include "Spi.h"
#include "Buffer.h"
#include "Uart.h"
#include "Sampling.h"
#include "AdcManager.h"
#include "Command.h"


// Sampling Parameters
volatile unsigned char triggerMode;
volatile char triggerLevel;
volatile unsigned char triggerEdge;
volatile unsigned char triggerChannel;
volatile unsigned char samplePeriodCode;
volatile unsigned char delayUnit;
volatile unsigned int delayTime;
volatile long numSampleData;
volatile long numSampleBlock;

volatile unsigned char periodCodeIndex;
volatile unsigned char samplingStatus;
#define		SAMPLING_STATUS_IDLE					0
#define		SAMPLING_STATUS_WAIT_TRIGGER			1
#define		SAMPLING_STATUS_WAIT_DELAY				2
#define		SAMPLING_STATUS_WAIT_COMPLETE			3
volatile unsigned char numDataPerInt;
volatile long numSampledRemain;
volatile long numSampledRemainBlock;
volatile long numAutoTriggerRemain;
volatile long numDelayRemain;
volatile unsigned char samplingSpeed;
#define SAMPLING_SPEED_FAST							0
#define SAMPLING_SPEED_SLOW							1
volatile int sendBlockSize;

unsigned char dataBuff[2][DATABUFF_SIZE * 2];
volatile int dataBufferIndex;
volatile int sendDataStartIndex;
volatile long seq_1;
volatile long seq_2;

//**********************************************************
//					Sampling_Init
//**********************************************************
void Sampling_Init()
{
	TRIGGER_INT_PORT |= TRIGGER_INT_MASK;
	TIMING_GEN_INT_PORT |= TIMING_GEN_INT_MASK;
	TIMING_GEN_CMND_DDR |= TIMING_GEN_CMND_MASK;
	TIMING_GEN_PARAM_DDR |= TIMING_GEN_PARAM_MASK;
}

//**********************************************************
//					SetTimingGeneratorCommand
//**********************************************************
void SetTimingGeneratorCommand(unsigned char cmnd, unsigned char param, unsigned char cont)
{
	TIMING_GEN_PARAM_PORT = (TIMING_GEN_PARAM_PORT & ~TIMING_GEN_PARAM_MASK) |
							(param << TIMING_GEN_PARAM_POS);
	TIMING_GEN_CMND_PORT = 	(TIMING_GEN_CMND_PORT & ~TIMING_GEN_CMND_MASK) | 
							(cmnd << TIMING_GEN_COMMAND_POS) ;
	TIMING_GEN_CMND_PORT |= _BV(TIMING_GEN_DO_COMMAND);
	if(cont == 0)
	{
		for(volatile char loop1 = 0; loop1 < 20; loop1++);
		TIMING_GEN_CMND_PORT &= ~_BV(TIMING_GEN_DO_COMMAND);
		for(volatile char loop2 = 0; loop2 < 10; loop2++);
	}
}

//**********************************************************
//					GetNumADCsCode
//**********************************************************
unsigned char GetNumADCsCode(unsigned char num)
{
	unsigned char code = 0;
	switch(num)
	{
		case 1:
			code = 0;
			break;
		case 2:
		case 3:
			code = 1;
			break;
		case 4:
		case 5:
		case 6:
		case 7:
			code = 2;
			break;
		case 8:
		default:
			code = 3;
			break;
	}
	return code;
}


//**********************************************************
//					SetTimingGenerator
//**********************************************************
void SetTimingGenerator(unsigned char periodCode, unsigned char numAdc)
{
	// Set Num ADCs
	SetTimingGeneratorCommand(TIMING_GEN_CMND_SET_ADCS, GetNumADCsCode(numAdc), 0);

	// Set Sample Period
	unsigned char cmnd = periodCode & 0x0F;
	unsigned char param = (periodCode >> 4) & 0x07;
	switch(param)
	{
	case 1:
		param = TIMING_GEN_PARAM_PERIOD_1;
		break;
	case 2:
		param = TIMING_GEN_PARAM_PERIOD_2;
		break;
	case 5:
	default:
		param = TIMING_GEN_PARAM_PERIOD_5;
		break;
	}
	SetTimingGeneratorCommand(cmnd, param, 1);
}

//**********************************************************
//					SetTriggerConditions
//**********************************************************
void SetTriggerConditions()
{
	unsigned char tmode = 0;
	if((triggerChannel) && (triggerEdge))		// triggerMode
	{
		if(triggerEdge == TRIGGER_MODE_PLUS)	// check slope
		{
			tmode = 0x10;
		} else {
			tmode = 0x11;
		}
		if(triggerMode == TRIGGER_MODE_AUTO)
		{
			numAutoTriggerRemain = GetAutoTriggerRemain(periodCodeIndex) / numDataPerInt;
		} else {
			numAutoTriggerRemain = GetAutoTriggerInfinite(periodCodeIndex) / numDataPerInt;
		}
		if(triggerChannel == 1) {				// Trigger CH1
			SetAdcTriggerMode(1, tmode);
			SetAdcTriggerLevel(1, triggerLevel);
			SetAdcTriggerMode(2, 0);
			EICRA = _BV(ISC01);					// Falling Edge
			EIMSK = _BV(INT0);
		} else {								// Trigger CH2
			SetAdcTriggerMode(1, 0);
			SetAdcTriggerMode(2, tmode);
			SetAdcTriggerLevel(2, triggerLevel);
			EICRA = _BV(ISC11);					// Falling Edge
			EIMSK = _BV(INT1);
		}
		samplingStatus = SAMPLING_STATUS_WAIT_TRIGGER;
	} else {
		// No Trigger
		SetAdcTriggerMode(1, 0);
		SetAdcTriggerMode(2, 0);
		samplingStatus = SAMPLING_STATUS_WAIT_COMPLETE;
	}
}

//**********************************************************
//					SetDelayedCondition
//**********************************************************
void SetDelayedCondition()
{
	if(delayTime != 0)
	{
		numDelayRemain = delayTime;					// uS
		if(delayUnit == 3) numDelayRemain *= 1000;	// mS
		if(delayUnit == 4) numDelayRemain *= 1000;	// S
		if(numDelayRemain > 10000000) numDelayRemain = 10000000; // 10 Sec
		numDelayRemain = TimebaseDivision(numDelayRemain, samplePeriodCode) / numDataPerInt;
	} else {
		numDelayRemain = 0;
	}
}

//**********************************************************
//					SamplingStart
//**********************************************************
void SamplingStart()
{
	int wasteCount = 0;
	cli();
	SamplingStop(STOP_REQUEST_OFF);
	samplingStatus = SAMPLING_STATUS_IDLE;
	fastSamplingCompleteIntFlag = INTERRUPT_NONE;
	slowSamplingIntFlag = INTERRUPT_NONE;
	periodCodeIndex = GetPeriodCodeIndex(samplePeriodCode);

	if((samplePeriodCode & 0x0F) < 8)	// < 100uS
	{
		numDataPerInt = GetNumADCs();
		samplingSpeed = SAMPLING_SPEED_FAST;
		sendBlockSize = DATABUFF_SIZE;
		if(numSampleData > GetBuffSize() * GetNumADCs())
		{
			numSampleData = GetBuffSize() * GetNumADCs();
		}
		wasteCount = 2;
	}
	else
	{
		numDataPerInt = 1;
		samplingSpeed = SAMPLING_SPEED_SLOW;
		dataBufferIndex = -1;
		sendDataStartIndex = 0;
		if((samplePeriodCode & 0x0F) < 9)	// < 1mS
		{
			sendBlockSize = DATABUFF_SIZE;
		}
		else
		{
			sendBlockSize = TimebaseCodeDivision(TIMEBASE_CODE_100mS, samplePeriodCode);
			if(sendBlockSize > DATABUFF_SIZE) sendBlockSize = DATABUFF_SIZE;
			if(sendBlockSize < 2) sendBlockSize = 1;
		}
	}
	numSampledRemain = (numSampleData + sendBlockSize - 1) / sendBlockSize;
	numSampledRemain = numSampledRemain * sendBlockSize;
	numSampleData = numSampledRemain;
	numSampledRemainBlock = numSampledRemain / numDataPerInt;
	numSampleBlock = numSampledRemainBlock;
	numSampledRemainBlock += wasteCount;
	seq_1 = seq_2 = 0;

	ADC_CLEAR_ALL;
	SetTriggerConditions();
	SetDelayedCondition();
	stopRequestFlag = STOP_REQUEST_OFF;
	EICRA |= _BV(ISC21);				// Falling Edge-
	EIMSK |= _BV(INT2);					// Start Sampling Count
	SetTimingGenerator(samplePeriodCode, GetNumADCs());
	sei();
}


//**********************************************************
//					SamplingStop
//**********************************************************
void SamplingStop(unsigned char forced)
{
	if(forced == STOP_REQUEST_ON)
	{
		stopRequestFlag = STOP_REQUEST_ON;
	}
	TIMING_GEN_CMND_PORT &= ~_BV(TIMING_GEN_DO_COMMAND);
	EIMSK &= ~_BV(INT2);
	for(volatile int loop1 = 0; loop1 < 50; loop1++);
}


//**********************************************************
//
//					External Interrupt vector
//
//	INT0: Trigger Souce CH.1
//		Trigger Condition Found
//
//	INT1: Trigger Source CH.2
//		Trigger Condition Found
//
//	INT2:
//		Sample Period50uSȉ̎A
//			SamplePeriod~NumADCsԊuŊ
//		SamplePeriod100uSȏ̂ƂA
//			SamplePeriodԊuŊ
//
//
//**********************************************************
ISR(INT0_vect)		// Check for auto trigger timeout
{
	if(samplingStatus == SAMPLING_STATUS_WAIT_TRIGGER)
	{
		if(numDelayRemain == 0)
		{
			samplingStatus = SAMPLING_STATUS_WAIT_COMPLETE;
		} 
		else 
		{
			samplingStatus = SAMPLING_STATUS_WAIT_DELAY;
		}
	}
	EIMSK &= ~_BV(INT0);
}

//--------------------------------------------------------------------
ISR(INT1_vect)
{
	if(samplingStatus == SAMPLING_STATUS_WAIT_TRIGGER)
	{
		if(delayTime == 0)
		{
			if(samplingSpeed == SAMPLING_SPEED_SLOW)
			{
				delayTime = 1;
				samplingStatus = SAMPLING_STATUS_WAIT_DELAY;
			}
			else
			{
				samplingStatus = SAMPLING_STATUS_WAIT_COMPLETE;
			}
		} else {
			samplingStatus = SAMPLING_STATUS_WAIT_DELAY;
		}
	}
	EIMSK &= ~_BV(INT1);
}

//--------------------------------------------------------------------
ISR(INT2_vect)
{
	if(samplingStatus == SAMPLING_STATUS_WAIT_COMPLETE)
	{
		if(samplingSpeed == SAMPLING_SPEED_SLOW)
		{
			slowSamplingIntFlag = INTERRUPT_OCCURED;
		} 
		else 
		{		// SAMPLING_SPEED_FAST
			numSampledRemainBlock--;
			if(numSampledRemainBlock == 0)
			{
				samplingStatus = SAMPLING_STATUS_IDLE;
				SamplingStop(STOP_REQUEST_OFF);
				fastSamplingCompleteIntFlag = INTERRUPT_OCCURED;
			}
		}
	}

	if(samplingStatus == SAMPLING_STATUS_WAIT_DELAY)
	{
		if(numDelayRemain-- <= 0)
		{
			samplingStatus = SAMPLING_STATUS_WAIT_COMPLETE;
		}
	}

	if(samplingStatus == SAMPLING_STATUS_WAIT_TRIGGER)
	{
		if(numAutoTriggerRemain-- <= 0)
		{
			if(numDelayRemain == 0)
			{
				samplingStatus = SAMPLING_STATUS_WAIT_COMPLETE;
			} 
			else 
			{
				samplingStatus = SAMPLING_STATUS_WAIT_DELAY;
			}
		}
	}
}

//**********************************************************
//					SlowSamplingDataTransfer
//**********************************************************
void SlowSamplingDataTransfer()
{
	txBuffType* sPtr;
	int sendSize;

	if(dataBufferIndex >= 0)
	{
		dataBuff[0][dataBufferIndex] = SPITransmit_CH1(0, ADC_COMMAND_CODE_NOP, SPI_TRANSIMIT_TERMINATE);
		dataBuff[1][dataBufferIndex] = SPITransmit_CH2(0, ADC_COMMAND_CODE_NOP, SPI_TRANSIMIT_TERMINATE);

		dataBufferIndex++;
		if(dataBufferIndex >= (sendBlockSize * 2)) 
		{
			dataBufferIndex = 0;
		}
		numSampledRemain--;
		if((numSampledRemain == 0) || (stopRequestFlag == STOP_REQUEST_ON))
		{
			samplingStatus = SAMPLING_STATUS_IDLE;
			SamplingStop(STOP_REQUEST_OFF);
		}

		if((dataBufferIndex == 0) || (dataBufferIndex == sendBlockSize) || (numSampledRemain == 0))
		{
			if(dataBufferIndex == 0)
			{
				sendSize = sendBlockSize;
			} 
			else 
			{
				sendSize = dataBufferIndex - sendDataStartIndex;
			}
			sPtr = GetFreeTxBuff(GET_BUFFER_WAIT);
			MakeReplySampling(sPtr->buff, CHANNEL_CODE_1, seq_1, sendBlockSize, &dataBuff[0][sendDataStartIndex]);
			SendTxBuff(sPtr);
			seq_1 += sendBlockSize;
			sPtr = GetFreeTxBuff(GET_BUFFER_WAIT);
			MakeReplySampling(sPtr->buff, CHANNEL_CODE_2, seq_2, sendBlockSize, &dataBuff[1][sendDataStartIndex]);
			SendTxBuff(sPtr);
			seq_2 += sendBlockSize;

			sendDataStartIndex = (sendDataStartIndex == 0)? sendBlockSize:0 ;
		}
	} 
	else 
	{
		dataBufferIndex = 0;
	}
}

//**********************************************************
//					FastSamplingDataTransfer
//**********************************************************
void FastSamplingDataTransfer()
{
	txBuffType* sPtr;
	unsigned char subIndex = 0;
	unsigned char unitIndex= 0;
	unsigned int numSampled = GetAdcNumData(0, 0);

	ADC_FETCH_FIRST_ALL;	// ǂݏo|C^̏
	if(numSampled < (GetBuffSize() + 2))
	{
		ADC_FETCH_NEXT_ALL;		// 擪f[^̓ǂݔ΂
		ADC_FETCH_NEXT_ALL;		// 擪f[^̓ǂݔ΂
		numSampled -= 2;
	}

	while(numSampled > numSampleBlock)
	{
		numSampled--;
		ADC_FETCH_NEXT_ALL;		// 擪f[^̓ǂݔ΂
	}

	for(long index = 0; index < numSampleData; index++)
	{
		if(stopRequestFlag == STOP_REQUEST_ON) break;

		dataBuff[0][subIndex] = SPITransmit_CH1(unitIndex, 
									ADC_COMMAND_CODE_FETCH_NEXT, 
									SPI_TRANSIMIT_TERMINATE);
		dataBuff[1][subIndex] = SPITransmit_CH2(unitIndex, 
									ADC_COMMAND_CODE_FETCH_NEXT, 
									SPI_TRANSIMIT_TERMINATE);
		unitIndex++;
		if(unitIndex >= GetNumADCs())
		{
			unitIndex = 0;
		}
		subIndex++;
		if(subIndex >= DATABUFF_SIZE)	// ]ubN𖞂H
		{
			subIndex = 0;
			sPtr = GetFreeTxBuff(GET_BUFFER_WAIT);	// Mobt@擾āA
			MakeReplySampling(sPtr->buff, CHANNEL_CODE_1, seq_1,	// bZ[W쐬
								DATABUFF_SIZE, dataBuff[0]);
			SendTxBuff(sPtr);						// M҂L[ɓ
			seq_1 += DATABUFF_SIZE;

			sPtr = GetFreeTxBuff(GET_BUFFER_WAIT);	// Mobt@擾āA
			MakeReplySampling(sPtr->buff, CHANNEL_CODE_2, seq_2, 	// bZ[W쐬A
								DATABUFF_SIZE, dataBuff[1]);
			SendTxBuff(sPtr);						// M҂L[ɓ
			seq_2 += DATABUFF_SIZE;
		}

		do {
			if(usartRxCompleteFlag == INTERRUPT_OCCURED)
			{
				usartRxCompleteFlag = INTERRUPT_NONE;
				RecieveDataReady();
			}
			if(usartUDREmptyFlag == INTERRUPT_OCCURED)
			{
				usartUDREmptyFlag = INTERRUPT_NONE;
				TransmitDataEmpty();
			}
			if(!IsUsedRxBuffEmpty())
			{
				DoCommand();
			}
		}
		// Mobt@Qȏ󂭂܂ő҂
		while((freeTxBuffRoot == 0) || (freeTxBuffRoot->next == 0));
	}
}
